﻿using System.Security;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using SoapEndpoint.Service.Proxy.Infrastructure;
using SoapEndpoint.Tester.Infrastructure;
using static SoapEndpoint.Tester.Infrastructure.FormHelperMethods;

namespace SoapEndpoint.Tester;

public partial class SimpleModeForm : Form
{
    private static readonly string CertificateDirPath = $"{Application.StartupPath}\\Certificate";
    private static readonly DirectoryInfo CertificateDir = new(CertificateDirPath);

    private readonly Version _defaultSystemVersion =
        SystemVersions.DefaultVersion(Program.Configuration["DefaultVersion"]);

    private ServiceUrlInfo _urlInfo;

    public SimpleModeForm()
    {
        InitializeComponent();
    }

    private FileInfo GetCertificateFile()
    {
        if (CertificateDir.Exists)
        {
            var file = CertificateDir.GetFiles("*.pfx").FirstOrDefault();
            return file;
        }

        return null;
    }

    private string GetCertificatePassword(string certificateFilePath)
    {
        var certificateFile = new FileInfo(certificateFilePath);
        if (certificateFile?.Exists == true)
        {
            var passwordFilePath = Path.ChangeExtension(certificateFile.FullName, ".txt");
            var passwordFile = new FileInfo(passwordFilePath);

            if (passwordFile.Exists)
            {
                return File.ReadAllText(passwordFile.FullName);
            }
        }

        return string.Empty;
    }

    private X509Certificate2 GetCertificate(string certificatePath, string certificatePassword)
    {
        try
        {
            var file = new FileInfo(certificatePath);
            if (file.Exists)
            {
                var cert = new X509Certificate2();
                var sec_pass = new SecureString();
                Array.ForEach(certificatePassword.ToArray(), sec_pass.AppendChar);
                sec_pass.MakeReadOnly();
                cert.Import(file.FullName, sec_pass, X509KeyStorageFlags.UserKeySet);
                return cert;
            }

            AddMessage("Het opgegeven certificaat bestaat niet", false);
        }
        catch (CryptographicException ex)
        {
            AddMessage($"Fout bij het ophalen van het certificaat. {ex.Message}", false);
        }
        catch (Exception ex)
        {
            AddMessage($"Fout bij het ophalen van het certificaat: {ex}", false);
        }

        return null;
    }

    private void SimpleModeForm_Load(object sender, EventArgs e)
    {
        var mapper = AutoMapperHelper.InitializeAutoMapper();

        _urlInfo = new ServiceUrlInfo(mapper);
        Environments.InitializeEnvironments();

        SetValues(GetCertificateFile()?.FullName);
    }

    private void SetValues(string certificateFilePath)
    {
        if (!string.IsNullOrWhiteSpace(certificateFilePath))
        {
            txtCertificatePath.Text = certificateFilePath;
            txtPassword.Text = GetCertificatePassword(certificateFilePath);
        }

        lblEnvironment.Text = Environments.DefaultEnvironment.Name;
        lblVersion.Text = _defaultSystemVersion?.ToString();
    }

    private async void button1_Click(object sender, EventArgs e)
    {
        await CheckConnectionAsync();
    }

    public void AddMessage(string message, bool success = true)
    {
        AddColoredMessage(rtbResult, message, success);
    }

    private async Task CheckConnectionAsync()
    {
        var certificate = GetCertificate(txtCertificatePath.Text, txtPassword.Text);
        if (certificate == null)
        {
            return;
        }

        try
        {
            var result = await _urlInfo
                .GetConnectionTestProxy(certificate, Environments.DefaultEnvironment, _defaultSystemVersion)
                .TestConnectionAsync();

            if (result)
            {
                AddMessage("Verbinding succesvol tot stand gebracht");
            }
            else
            {
                AddMessage("Fout bij het maken van de verbinding", false);
            }
        }
        catch (Exception ex)
        {
            AddMessage($"Fout bij het maken van de verbinding: {ex}", false);
        }
    }

    private void btnBrowseCertificateFile_Click(object sender, EventArgs e)
    {
        var dialog = new OpenFileDialog
        {
            InitialDirectory = CertificateDirPath,
            Filter = "Certificaat met private key|*.pfx"
        };

        var result = dialog.ShowDialog();
        if (result == DialogResult.OK)
        {
            SetValues(dialog.FileName);
        }
    }

    private void cbShowPass_CheckedChanged(object sender, EventArgs e)
    {
        txtPassword.PasswordChar = !cbShowPass.Checked ? '●' : (char)0;
    }
}